@@ -25,13 +25,17 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
-#include <math.h>
 #include <string.h>
+/*#include <gst/gst.h>*/
 
 #include "minefield.h"
 #include "games-clock.h"
 #include "games-frame.h"
+#include "games-scores.c"
+#include "games-scores-dialog.h"
+#include "games-stock.h"
 
 /* Limits for various minefield properties */
 #define XSIZE_MIN 4
@@ -51,56 +55,61 @@
 #define KEY_WIDTH    	       "/apps/gnomine/geometry/width"
 #define KEY_HEIGHT   	       "/apps/gnomine/geometry/height"
 
-/* So that it is only one place if we re-order the menus. */
-#define HINT_MENU_LOC 1
-
-static GtkWidget *mf_frame;
 static GtkWidget *mfield;
 static GtkWidget *pref_dialog = NULL;
-static GtkWidget *rbutton;
-static GtkWidget *ralign;
+static GtkWidget *resume_button;
+static GtkWidget *resume_container;
+static GtkWidget *mfield_container;
 static GConfClient *conf_client;
 GtkWidget *window;
 GtkWidget *flabel;
-GtkWidget *xentry;
-GtkWidget *yentry;
 GtkWidget *mentry;
-GtkWidget *sentry;
-GtkWidget *question_toggle;
 GtkWidget *mbutton;
-GtkWidget *plabel;
 GtkWidget *cframe;
 GtkWidget *clk;
 GtkWidget *pm_win, *pm_sad, *pm_smile, *pm_cool, *pm_worried, *pm_current;
-GtkWidget *face_box;
 gint ysize = -1, xsize = -1;
 gint nmines = -1;
 gint fsize = -1;
 gboolean use_question_marks = TRUE;
 
-GnomeUIInfo gamemenu[];
-GnomeUIInfo settingsmenu[];
-GnomeUIInfo helpmenu[];
-GnomeUIInfo mainmenu[];
+GtkAction *hint_action;
+GtkAction *fullscreen_action;
+GtkAction *leavefullscreen_action;
 
-char *fsize2names[] = {
-	N_("Small"),
-	N_("Medium"),
-	N_("Large"),
-	N_("Custom"),
-};
+/*GstElement *sound_player;*/
+
+static const GamesScoresCategory scorecats[] = {{"Small", N_("Small")},
+						{"Medium", N_("gnomine|Medium")},
+						{"Large", N_("Large")},
+						{"Custom", N_("Custom")},
+						GAMES_SCORES_LAST_CATEGORY};
+
+static const GamesScoresDescription scoredesc = {scorecats,
+						 "Small",
+						 "gnomine",
+						 GAMES_SCORES_STYLE_TIME_ASCENDING};
+
+GamesScores *highscores;
 
 /* It's a little ugly, but it stops the hint dialogs triggering the
  * hide-the-window-to-stop cheating thing. */
 gboolean disable_hiding = FALSE;
 
-static void create_preferences (void);
+#if 0
+static void
+play_sound (int id)
+{
+	/* FIXME: We ignore the id */
 
+	/* To play, do this: */
+	gst_element_set_state (sound_player, GST_STATE_PLAYING);	
+}
+#endif
 
 static GtkWidget *
 image_widget_setup (char *name)
 {
-	/*	GdkPixbuf *pixbuf = NULL; */
 	GtkWidget *image = NULL;
 	char *filename = NULL;
 
@@ -110,11 +119,9 @@ image_widget_setup (char *name)
 					      TRUE, NULL);
 	if (filename != NULL)
 		gtk_image_set_from_file (GTK_IMAGE (image), filename);
-	/*		pixbuf = gdk_pixbuf_new_from_file (filename, NULL); */
 
 	g_free (filename);
 
-	/*	return gtk_image_new_from_pixbuf (pixbuf); */
 	return image;
 }
 
@@ -126,17 +133,15 @@ show_face (GtkWidget *pm)
 	if (pm_current) {
 		gtk_widget_hide (pm_current); 
 	}
-
-	gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget, 
+	gtk_action_set_sensitive(hint_action,
 				 (pm == pm_cool) || (pm == pm_smile));
-
 	gtk_widget_show (pm);
 	
 	pm_current = pm;
 }
 
 static void
-quit_game (GtkWidget *widget, gpointer data)
+quit_game (void)
 {
 	gint width, height;
 
@@ -152,193 +157,96 @@ set_flabel (GtkMineField *mfield)
 {
 	char *val;
 
-	val = g_strdup_printf ("%d/%d", mfield->flag_count, mfield->mcount);
+	val = g_strdup_printf (_("Flags: %d/%d"), mfield->flag_count, mfield->mcount);
 	gtk_label_set_text (GTK_LABEL (flabel), val);
 	g_free (val);
 }
 
-static void
-update_score_state (void)
+/* Show the high scores dialog - creating it if necessary. If pos is 
+ * greater than 0 the appropriate score is highlighted. If the score isn't
+ * a high score and this isn't a direct request to see the scores, we 
+ * only show a simple dialog. */
+static gint
+show_scores (gint pos, gboolean endofgame)
 {
-        gchar **names = NULL;
-        gfloat *scores = NULL;
-	time_t *scoretimes = NULL;
-	gint top;
-	gchar buf[64];
-
-	if (fsize<4)
-		strncpy (buf, fsize2names[fsize], sizeof(buf));
-
-	top = gnome_score_get_notable ("gnomine", buf, &names, &scores, &scoretimes);
-	if (top > 0) {
-		gtk_widget_set_sensitive (gamemenu[2].widget, TRUE);
-		g_strfreev (names);
-		g_free (scores);
-		g_free (scoretimes);
-	} else { 
-		gtk_widget_set_sensitive (gamemenu[2].widget, FALSE);
-	}
-}
-
-static void 
-fill_score_list (GtkListStore *list, gint level)
-{
-	int i,n;
-	GtkTreeIter iter;
-	gchar **names = NULL;
-	gfloat *scores = NULL;
-	time_t *scoretimes = NULL;
-	gchar **name;
-	gfloat *score;
-	gchar * ss;
-
-	gtk_list_store_clear (list);
-
-	n = gnome_score_get_notable ("gnomine", fsize2names[level], &names, 
-				     &scores, &scoretimes);
-
-	i = 1;
-	name = names;
-	score = scores;
-	while (n--) {
-		int intscore;
-
-		intscore = rint (100*(*score));
-		/* Translators: this is for a minutes, seconds time display. */
-		ss = g_strdup_printf (_("%dm %ds"), intscore/100, intscore%100);
-		gtk_list_store_append (list, &iter);
-		gtk_list_store_set (list, &iter, 0, *name, 1, ss, -1);
-		g_free (ss);
-		name++;
-		score++;
-	}
-	
-	g_free (names);
-	g_free (scores);
-	g_free (scoretimes);
-}
-
-static void 
-change_score_display (GtkWidget *widget, GtkListStore *list)
-{
-	fill_score_list (list, 
-			 gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
-}
-
-static void
-show_scores (gchar *level, guint pos)
-{
-	int i, n;
+	gchar *message;
+	static GtkWidget *scoresdialog = NULL;
+	static GtkWidget *sorrydialog = NULL;
 	GtkWidget *dialog;
-	GtkWidget *hbox;
-	GtkWidget *vbox;
-	GtkWidget *combo;
-	GtkWidget *scroll;
-	GtkWidget *listview;
-	GtkTreeViewColumn *column;
-        GtkTreeSelection * select;
-        GtkListStore *list;
-	GtkCellRenderer *renderer;
-	GtkTreePath * path;
-	GtkWidget * label;
-	gchar *lstr;
-	GtkWidget * button;
+	gint result;
 
-	dialog = gtk_dialog_new_with_buttons (_("GNOME Mines Scores"),
-					      GTK_WINDOW (window), 
-					      GTK_DIALOG_DESTROY_WITH_PARENT |
-					      GTK_DIALOG_NO_SEPARATOR,
-					      NULL);
-	/* FIXME: There has to be an easier way to force the default
-	 * widget. */
-	button = gtk_dialog_add_button (GTK_DIALOG (dialog), 
-					GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT);
-	gtk_widget_grab_focus (button);
+	if (endofgame && (pos <= 0)) {
+		if (sorrydialog != NULL) {
+			gtk_window_present (GTK_WINDOW (sorrydialog));
+		} else {
+			sorrydialog = gtk_message_dialog_new_with_markup (
+					  GTK_WINDOW (window),
+					  GTK_DIALOG_DESTROY_WITH_PARENT,
+					  GTK_MESSAGE_INFO,
+					  GTK_BUTTONS_NONE, 
+					  "<b>%s</b>\n%s", 
+					  _("The Mines Have Been Cleared!"), 
+					  _("Great work, but unfortunately your score did not make the top ten."));
+			gtk_dialog_add_buttons (GTK_DIALOG (sorrydialog),
+						GTK_STOCK_QUIT, GTK_RESPONSE_REJECT,
+						_("_New Game"),
+						GTK_RESPONSE_ACCEPT,
+						NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (sorrydialog),
+							 GTK_RESPONSE_ACCEPT);
+			gtk_window_set_title (GTK_WINDOW (sorrydialog), "");
+		}
+		dialog = sorrydialog;
+	} else {
 
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
+		if (scoresdialog != NULL) {
+			gtk_window_present (GTK_WINDOW (scoresdialog));
+		} else {
+			scoresdialog = games_scores_dialog_new (highscores, 
+								_("Mines Scores"));
+			games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG (scoresdialog),
+								      _("Size:"));	
+		}
+		
+		if (pos > 0) {
+			games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (scoresdialog), 
+							 pos);
+			message = g_strdup_printf ("<b>%s</b>\n\n%s", 
+						   _("Congratulations!"),
+						   _("Your score has made the top ten."));
+			games_scores_dialog_set_message (GAMES_SCORES_DIALOG (scoresdialog),
+							 message);
+			g_free (message);
+		} else {
+			games_scores_dialog_set_message (GAMES_SCORES_DIALOG (scoresdialog),
+							 NULL);
+		}
 
-	hbox = gtk_hbox_new (FALSE, 5);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
-
-	lstr = g_strdup_printf ("<b>%s</b>", _("Size:"));
-	label = gtk_label_new (lstr);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	g_free (lstr);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);	
-
-	combo = gtk_combo_box_new_text ();
-	gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 5);
-	n = 0;
-	for (i=0; i<4; i++) {
-		if (g_utf8_collate (level, fsize2names[i]) == 0)
-			n = i;
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-					   _(fsize2names[i]));
+		if (endofgame) {
+			games_scores_dialog_set_buttons (GAMES_SCORES_DIALOG (scoresdialog),
+							 GAMES_SCORES_QUIT_BUTTON |
+							 GAMES_SCORES_NEW_GAME_BUTTON);
+		} else {
+			games_scores_dialog_set_buttons (GAMES_SCORES_DIALOG (scoresdialog),
+							 0);
+		}
+		dialog = scoresdialog;
 	}
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), n);
+		
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_hide (dialog);
 
-	scroll = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_size_request (scroll, 250, 265);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
-					      GTK_SHADOW_ETCHED_IN);
-	gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
-
-	list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-
-	g_signal_connect (G_OBJECT (combo), "changed", 
-			  G_CALLBACK (change_score_display), list);
-
-	fill_score_list (list, n);
-
-        listview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list));
-	renderer = gtk_cell_renderer_text_new ();
-        column = gtk_tree_view_column_new_with_attributes (_("Name"),
-                                                           renderer,
-                                                           "text", 0,
-                                                           NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (listview),
-                                     GTK_TREE_VIEW_COLUMN (column));
-	renderer = gtk_cell_renderer_text_new ();
-        column = gtk_tree_view_column_new_with_attributes (_("Time"),
-                                                           renderer,
-                                                           "text", 1,
-                                                           NULL);
-	g_object_set (G_OBJECT (renderer), "xalign", 1.0, NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (listview),
-                                     GTK_TREE_VIEW_COLUMN (column));
-
-        select = gtk_tree_view_get_selection (GTK_TREE_VIEW (listview));
-	if (pos > 0) {
-		path = gtk_tree_path_new_from_indices (pos-1, -1);
-		gtk_tree_selection_select_path (select, path);
-	}
-	                                      
-	gtk_container_add (GTK_CONTAINER (scroll), listview);
-
-	gtk_widget_show_all (dialog);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-
-	gtk_widget_destroy (dialog);
+	return result;
 }
 
 static void
-top_ten (GtkWidget *widget, gpointer data)
+scores_callback (void)
 {
-	gchar buf[64];
-
-	if(fsize<4)
-		strncpy (buf, fsize2names[fsize], sizeof(buf));
-
-	show_scores (buf, 0);
+	show_scores (0, FALSE);
 }
 
 static void
-new_game (GtkWidget *widget, gpointer data)
+new_game (void)
 {
 	gint width, height, w_diff, h_diff;
 	guint size;
@@ -365,13 +273,14 @@ new_game (GtkWidget *widget, gpointer data)
 		y = size_table[fsize][1];
 		mf->mcount = size_table[fsize][2];
 	}
+	games_scores_set_category (highscores, scorecats[fsize].key);
 	gtk_minefield_set_size (GTK_MINEFIELD (mfield), x, y);
 	gtk_minefield_restart (GTK_MINEFIELD (mfield));
 
 	set_flabel (GTK_MINEFIELD (mfield));
 
-	gtk_widget_hide (ralign);
-	gtk_widget_show (mf_frame); 
+	gtk_widget_hide (resume_container);
+	gtk_widget_show (mfield_container); 
 }
 
 /* Add a penalty for a successful hint. */
@@ -383,7 +292,7 @@ hint_used (GtkWidget *widget, gpointer data)
 }
 
 static void
-hint (GtkWidget *widget, gpointer data)
+hint_callback (void)
 {
 	int result;
 	gchar * message;
@@ -418,11 +327,11 @@ focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
 	if ((GAMES_CLOCK (clk)->timer_id != -1) 
 	    && (!disable_hiding)) {
-		gtk_widget_hide (mf_frame);
-		gtk_widget_show (ralign);
-		gtk_widget_grab_focus (rbutton);
-		gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget,FALSE);
-
+		gtk_widget_hide (mfield_container);
+		gtk_widget_show (resume_container);
+		gtk_widget_grab_focus (resume_button);
+		
+		gtk_action_set_sensitive (hint_action, FALSE);
 		games_clock_stop (GAMES_CLOCK (clk)); 
 	}
 }
@@ -430,10 +339,9 @@ focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 static void
 resume_game_cb (GtkButton *widget, gpointer data)
 {
-	gtk_widget_hide (ralign);
-	gtk_widget_show (mf_frame);
-	gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget,TRUE);
-
+	gtk_widget_hide (resume_container);
+	gtk_widget_show (mfield_container);
+	gtk_action_set_sensitive (hint_action, TRUE);
 	games_clock_start (GAMES_CLOCK (clk));
 }
 
@@ -458,10 +366,8 @@ lose_game (GtkWidget *widget, gpointer data)
 static void
 win_game (GtkWidget *widget, gpointer data)
 {
-        gfloat score;
+        gdouble score;
         int pos;
-        gchar buf[64];
-
 
         games_clock_stop (GAMES_CLOCK (clk));
 
@@ -473,19 +379,20 @@ win_game (GtkWidget *widget, gpointer data)
 	    score = (gfloat) (GAMES_CLOCK (clk)->stopped / 60) + 
 		    (gfloat) (GAMES_CLOCK (clk)->stopped % 60) / 100;
 
-            strncpy(buf, fsize2names[fsize], sizeof(buf));
-	    pos = gnome_score_log (score, buf, FALSE);
+	    pos = games_scores_add_score (highscores, (GamesScoreValue) score);
 	} else {
+	    /* FIXME: This is actually broken by the new high score code,
+	     * which interprets it as a time. */
 	    score = ((nmines * 100) / (xsize * ysize)) /
 		    (gfloat) (GAMES_CLOCK (clk)->stopped); 
 
-            strncpy(buf, fsize2names[fsize], sizeof(buf));
-	    pos = gnome_score_log (score, buf, TRUE);
+	    pos = games_scores_add_score (highscores, (GamesScoreValue) score);
 	}
 
-	update_score_state ();
-
-	show_scores (buf, pos);
+	if (show_scores (pos, TRUE) == GTK_RESPONSE_REJECT)
+		quit_game ();
+	else
+		new_game ();
 }
 
 static void
@@ -507,17 +414,14 @@ verify_ranges (void)
 {
 	xsize    = CLAMP (xsize, XSIZE_MIN, XSIZE_MAX);
 	ysize    = CLAMP (ysize, YSIZE_MIN, YSIZE_MAX);
-	nmines   = CLAMP (nmines, 1, xsize * ysize - 2);
+	nmines   = CLAMP (nmines, 1, xsize * ysize - 9);
 	fsize    = CLAMP (fsize,  0, 3);
 }
 
 static void
-about (GtkWidget *widget, gpointer data)
+about_callback (void)
 {
-        static GtkWidget *about;
-	GdkPixbuf *pixbuf = NULL;
-
-        const gchar *authors[] = {
+	const gchar *authors[] = {
 		_("Main game:"),
 		"Szekeres Istvan",
 		"",
@@ -535,49 +439,15 @@ about (GtkWidget *widget, gpointer data)
 		"Callum McKenzie",
 		NULL
 	};
-	const gchar *documenters[] = {
-                NULL
-        };
-	const gchar *translator_credits = _("translator-credits");
-	
-	if (about) {
-                gtk_window_present (GTK_WINDOW (about));
-		return;
-	}
 
-       {
-            int i=0;
-            while (authors[i] != NULL) { authors[i]=_(authors[i]); i++; }
-       }
-       {
-	       char *filename = NULL;
-
-	       filename = gnome_program_locate_file (NULL,
-			       GNOME_FILE_DOMAIN_APP_PIXMAP,  ("gnome-gnomine.png"),
-			       TRUE, NULL);
-	       if (filename != NULL)
-	       {
-		       pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
-		       g_free (filename);
-	       }
-       }
-
-        about = gnome_about_new (_("GNOME Mines"), VERSION,
-				 "Copyright \xc2\xa9 1997-2004 Free Software "
-				 "Foundation, Inc.",
-				 _("A Minesweeper clone."),
-				 (const char **)authors,
-				 (const char **)documenters,
-				 strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-	
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
-	
-	g_signal_connect (GTK_OBJECT (about), "destroy", GTK_SIGNAL_FUNC
-			(gtk_widget_destroyed), &about);
-	gtk_window_set_transient_for(GTK_WINDOW (about), GTK_WINDOW (window));
-        gtk_widget_show (about);
+	gtk_show_about_dialog (GTK_WINDOW(window),
+			       "name", _("Mines"),
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 1997-2004 Free Software Foundation, Inc.",
+			       "authors", authors,
+			       "translator_credits", _("translator-credits"),
+			       "logo-icon-name", "gnome-gnomine",
+			       NULL);
 }
 
 static void
@@ -595,7 +465,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 16;
 		if (i != xsize) {
 			xsize = CLAMP (i, XSIZE_MIN, XSIZE_MAX);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_YSIZE) == 0) {
@@ -603,7 +473,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 16;
 		if (i != ysize) {
 			ysize = CLAMP (i, YSIZE_MIN, YSIZE_MAX);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_NMINES) == 0) {
@@ -611,7 +481,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 40;
 		if (nmines != i) {
 			nmines = CLAMP (i, 1, xsize * ysize - 2);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_MODE) == 0) {
@@ -619,8 +489,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 0;
 		if (i != fsize) {
 			fsize = CLAMP (i, 0, 3);
-			update_score_state ();
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_USE_QUESTION_MARKS) == 0) {
@@ -649,10 +518,11 @@ fix_nmines (int xsize, int ysize)
 {
 	int maxmines;
 
-	/* Fix up the maximum number of mines so that there is always at least two
-	 * free spaces. It could in theory be left at one, but that gives an
-	 * instant win situation. */
-	maxmines = xsize*ysize - 2;
+	/* Fix up the maximum number of mines so that there is always at least
+	   ten free spaces. Nine are so we can clear at least the immediate
+	   eight neighbours at the start and one more so the game isn't over 
+	   immediately. */
+	maxmines = xsize*ysize - 10;
 	if (nmines > maxmines) {
 		gconf_client_set_int (conf_client, KEY_NMINES, maxmines, NULL);
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (mentry), maxmines);
@@ -695,6 +565,34 @@ use_question_toggle_cb (GtkCheckButton *check, gpointer data)
 				use_marks, NULL);
 }
 
+static void
+set_fullscreen_actions (gboolean is_fullscreen)
+{
+	gtk_action_set_sensitive (leavefullscreen_action, is_fullscreen);
+	gtk_action_set_visible (leavefullscreen_action, is_fullscreen);
+
+	gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
+	gtk_action_set_visible (fullscreen_action, !is_fullscreen);
+}
+
+static void
+fullscreen_callback (GtkAction *action)
+{
+	if (action == fullscreen_action)
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	else
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+}
+  	 
+static void
+window_state_callback (GtkWidget *widget, GdkEventWindowState *event)
+{
+	if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
+		return;
+
+	set_fullscreen_actions (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN);
+}
+
 static void
 create_preferences (void)
 {
@@ -704,16 +602,18 @@ create_preferences (void)
 	GtkWidget *button;
 	GtkWidget *table2;
 	GtkWidget *label2;
-        
-	table = gtk_table_new (2, 3, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD);
-	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD);
-	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD);
+	GtkWidget *question_toggle;        
+	GtkWidget *xentry;
+	GtkWidget *yentry;
 
-	frame = games_frame_new (_("Field size"));
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 18);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 18);
 
-	vbox = gtk_vbox_new (TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
+	frame = games_frame_new (_("Field Size"));
+
+	vbox = gtk_vbox_new (FALSE, 6);
 
 	button = gtk_radio_button_new_with_label(NULL, _("Small"));
 	if (fsize == 0)
@@ -725,7 +625,7 @@ create_preferences (void)
 
 	button = gtk_radio_button_new_with_label
 		(gtk_radio_button_get_group (GTK_RADIO_BUTTON(button)),
-		 _("Medium"));
+		 Q_("gnomine|Medium"));
 	if (fsize == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
@@ -751,79 +651,77 @@ create_preferences (void)
 				TRUE);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 3);
-	gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 
-	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1, GTK_EXPAND |
-			GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 1, 0, 1);
 
-	cframe = games_frame_new (_("Custom size"));
+	cframe = games_frame_new (_("Custom Size"));
 	gtk_widget_set_sensitive (cframe, fsize == 3);
 
 	table2 = gtk_table_new (3, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table2), GNOME_PAD);
-	gtk_table_set_row_spacings (GTK_TABLE (table2), GNOME_PAD);
+	gtk_table_set_row_spacings (GTK_TABLE (table2), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table2), 12);
 
-	label2 = gtk_label_new (_("Number of mines:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Number of mines:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 2, 3,
-			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	/* TODO: fix this when gconfised correctly */
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	
 	mentry = gtk_spin_button_new_with_range (1, XSIZE_MAX*YSIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (mentry), "value-changed",
 			GTK_SIGNAL_FUNC (nmines_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(mentry), nmines);
-	gtk_table_attach (GTK_TABLE (table2), mentry, 1, 2, 2, 3, GTK_FILL, GTK_EXPAND
-			| GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), mentry, 1, 2, 2, 3, 0, 0, 0, 0);
+	fix_nmines (xsize, ysize);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), mentry);
 
-	label2 = gtk_label_new (_("Horizontal:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Horizontal:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1,
-			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
 	xentry = gtk_spin_button_new_with_range (XSIZE_MIN, XSIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (xentry), "value-changed",
 			GTK_SIGNAL_FUNC (xsize_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(xentry), xsize);
-	gtk_table_attach (GTK_TABLE (table2), xentry, 1, 2, 0, 1, 0, GTK_EXPAND
-			| GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), xentry, 1, 2, 0, 1, 0, 0, 0, 0);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), xentry);
 
-	label2 = gtk_label_new (_("Vertical:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Vertical:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2,
-			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
 	yentry = gtk_spin_button_new_with_range (YSIZE_MIN, YSIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (yentry), "value-changed",
 			GTK_SIGNAL_FUNC (ysize_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(yentry), ysize);
-	gtk_table_attach (GTK_TABLE (table2), yentry, 1, 2, 1, 2, 0, GTK_EXPAND
-			| GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), yentry, 1, 2, 1, 2, 0, 0, 0, 0); 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), yentry);
 
 	gtk_container_add (GTK_CONTAINER (cframe), table2);
 
-	gtk_table_attach (GTK_TABLE (table), cframe, 1, 2, 0, 1, GTK_EXPAND |
-			GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), cframe, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0); 
 
-	question_toggle = gtk_check_button_new_with_label(_("Use \"I'm not sure\" flags."));
+	question_toggle = gtk_check_button_new_with_mnemonic(_("_Use \"I'm not sure\" flags"));
 	g_signal_connect(GTK_OBJECT(question_toggle), "toggled",
 			GTK_SIGNAL_FUNC (use_question_toggle_cb), NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(question_toggle), use_question_marks);
 	gtk_widget_show(question_toggle);
 	
-	gtk_table_attach (GTK_TABLE (table), question_toggle, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_table_attach_defaults (GTK_TABLE (table), question_toggle, 0, 2, 1, 2);
 
 
-	pref_dialog = gtk_dialog_new_with_buttons (_("GNOME Mines Preferences"),
+	pref_dialog = gtk_dialog_new_with_buttons (_("Mines Preferences"),
 			GTK_WINDOW (window),
 			0,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			NULL);
 
         gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
-	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(pref_dialog)->vbox),
-			table);
+	gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), 2);
+	gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(pref_dialog)->vbox), table, FALSE, FALSE, 0);
 
 	g_signal_connect (G_OBJECT (pref_dialog), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &pref_dialog);
@@ -835,40 +733,78 @@ create_preferences (void)
 }
 
 static void
-preferences_callback (GtkWidget *widget, gpointer data)
+preferences_callback (void)
 {
 	if (pref_dialog == NULL)
 		create_preferences ();
 	gtk_window_present (GTK_WINDOW (pref_dialog));
 }
 
-GnomeUIInfo gamemenu[] = {
-        GNOMEUIINFO_MENU_NEW_GAME_ITEM(new_game, NULL),
-	GNOMEUIINFO_MENU_HINT_ITEM(hint, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_SCORES_ITEM(top_ten, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_QUIT_ITEM(quit_game, NULL),
-	GNOMEUIINFO_END
+static void
+help_callback (void)
+{
+	gnome_help_display ("gnomine.xml", NULL, NULL);
+}
+
+const GtkActionEntry actions[] = {
+	{ "GameMenu", NULL, N_("_Game") },
+	{ "SettingsMenu", NULL, N_("_Settings") },
+	{ "HelpMenu", NULL, N_("_Help") },
+	{ "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_game) },
+	{ "Hint", GAMES_STOCK_HINT, NULL, NULL , NULL, G_CALLBACK (hint_callback) },
+	{ "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (scores_callback) },
+	{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_game) },
+	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (preferences_callback) },
+	{ "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_callback) },
+	{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_callback) },
+	{ "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) },
+	{ "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) }
 };
 
-GnomeUIInfo settingsmenu[] = {
-	GNOMEUIINFO_MENU_PREFERENCES_ITEM(preferences_callback, NULL),
-	GNOMEUIINFO_END
-};
+const char ui_description[] =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='GameMenu'>"
+"      <menuitem action='NewGame'/>"
+"      <menuitem action='Hint'/>"
+"      <separator/>"
+"      <menuitem action='Scores'/>"
+"      <separator/>"
+"      <menuitem action='Quit'/>"
+"    </menu>"
+"    <menu action='SettingsMenu'>"
+"      <menuitem action='Fullscreen'/>"
+"      <menuitem action='LeaveFullscreen'/>"
+"      <menuitem action='Preferences'/>"
+"    </menu>"
+"    <menu action='HelpMenu'>"
+"      <menuitem action='Contents'/>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
 
-GnomeUIInfo helpmenu[] = {
-        GNOMEUIINFO_HELP("gnomine"),
-	GNOMEUIINFO_MENU_ABOUT_ITEM(about, NULL),
-	GNOMEUIINFO_END
-};
+static GtkUIManager*
+create_ui_manager (const gchar *group)
+{
+	GtkActionGroup *action_group;
+	GtkUIManager *ui_manager;
+	
+	action_group = gtk_action_group_new ("group");
+	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions), window);
 
-GnomeUIInfo mainmenu[] = {
-        GNOMEUIINFO_MENU_GAME_TREE(gamemenu),
-	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
-        GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
-	GNOMEUIINFO_END
-};
+	ui_manager = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
+	hint_action = gtk_action_group_get_action (action_group, "Hint");
+
+	fullscreen_action = gtk_action_group_get_action (action_group, "Fullscreen");
+	leavefullscreen_action = gtk_action_group_get_action (action_group, "LeaveFullscreen");
+	set_fullscreen_actions (FALSE);
+
+	return ui_manager;
+}
 
 static int
 save_state (GnomeClient        *client,
@@ -923,6 +859,29 @@ static struct poptOption options[] = {
   {NULL, '\0', 0, NULL, 0}
 };
 
+#if 0
+static void 
+sound_eos (GstElement *sound_player, gpointer data)
+{
+	/* Reset the sound. */
+	gst_element_set_state (sound_player, GST_STATE_NULL);
+}
+
+static void 
+sound_init (int * argcp, char ** argvp[])
+{
+
+	gst_init (argcp, argvp);
+	sound_player = gst_element_factory_make ("playbin", "play");
+	g_object_set (G_OBJECT (sound_player), "uri", "file:///home/callum/Desktop/Development/CVS/gnome/gnome-games/gnometris/sounds/land.wav", NULL);
+
+	/* Attach the signal for end-of-stream. */
+	g_signal_connect (G_OBJECT (sound_player), "eos", G_CALLBACK (sound_eos), 
+			  NULL);
+
+}
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -931,10 +890,14 @@ main (int argc, char *argv[])
 	GtkWidget *button_table;
         GtkWidget *box;        
         GtkWidget *label;
+	GtkWidget *face_box;
 	GnomeClient *client;
+	GtkUIManager *ui_manager;
+	GtkAccelGroup *accel_group;
+
 	gint width, height;
 
-	gnome_score_init("gnomine");
+	setgid_io_init ();
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -946,7 +909,11 @@ main (int argc, char *argv[])
 			GNOME_PARAM_POPT_TABLE, options,
 			GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
-	/* Get the default GConfClient */
+	highscores = games_scores_new (&scoredesc);
+
+	/* sound_init (&argc, &argv); */
+
+	/* We hold one global reference to the default gconf client. */
 	conf_client = gconf_client_get_default ();
         gconf_client_add_dir (conf_client, 
                               KEY_DIR,
@@ -991,8 +958,9 @@ main (int argc, char *argv[])
 	 * to be the default). */
 	gtk_rc_parse_string ("style \"gnomine\" { GtkButton::interior-focus = 0 } class \"GtkButton\" style \"gnomine\"");
 
-	window = gnome_app_new ("gnomine", _("GNOME Mines"));
-	gnome_app_create_menus (GNOME_APP (window), mainmenu);
+	window = gnome_app_new ("gnomine", _("Mines"));
+	games_stock_init ();
+
 	gtk_window_set_default_size (GTK_WINDOW (window), width, height);	
 	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
 
@@ -1000,13 +968,20 @@ main (int argc, char *argv[])
 			 G_CALLBACK (quit_game), NULL);
 	g_signal_connect(G_OBJECT (window), "focus_out_event",
 			 G_CALLBACK (focus_out_cb), NULL);
+	g_signal_connect (G_OBJECT (window), "window_state_event",
+			  G_CALLBACK (window_state_callback), NULL);
 
 	all_boxes = gtk_vbox_new (FALSE, 0);
 
 	gnome_app_set_contents (GNOME_APP (window), all_boxes);
+        ui_manager = create_ui_manager ("GnomineActions");
+        accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+        gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+        box = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+        gtk_box_pack_start (GTK_BOX (all_boxes), box, FALSE, FALSE, 0);
 
 	button_table = gtk_table_new (1, 3, FALSE);
-	gtk_box_pack_start (GTK_BOX (all_boxes), button_table, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (all_boxes), button_table, TRUE, TRUE, 0);
 
 	pm_current = NULL;
 
@@ -1039,21 +1014,20 @@ main (int argc, char *argv[])
 
 	mfield = gtk_minefield_new ();
 
-	/* It doesn't really matter what this widget is as long as it's a
-	 * container. */
-	mf_frame = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-	gtk_container_add (GTK_CONTAINER (mf_frame), mfield);
+	/* These next two widgets are created as alignments, but it's more
+	 * important to know that these are containers, hence the names. */
+	mfield_container = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+	gtk_container_add (GTK_CONTAINER (mfield_container), mfield);
+	gtk_box_pack_start (GTK_BOX (box), mfield_container, TRUE, TRUE, 0);
 
-	gtk_box_pack_start (GTK_BOX (box), mf_frame, TRUE, TRUE, 0);
+	resume_container = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (box), resume_container, TRUE, TRUE, 0);
 
-	ralign = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (box), ralign, TRUE, TRUE, 0);
-
-	rbutton = gtk_button_new_with_label ("Press to resume");
-	g_signal_connect (G_OBJECT (rbutton), "clicked", 
+	resume_button = gtk_button_new_with_label (_("Press to Resume"));
+	g_signal_connect (G_OBJECT (resume_button), "clicked", 
 			  G_CALLBACK (resume_game_cb), NULL);
-        gtk_container_add (GTK_CONTAINER (ralign), rbutton);
-
+        gtk_container_add (GTK_CONTAINER (resume_container), resume_button);
+	
 	gtk_minefield_set_use_question_marks (GTK_MINEFIELD (mfield),
 					      use_question_marks);
 	
@@ -1077,25 +1051,23 @@ main (int argc, char *argv[])
 	gtk_box_pack_start (GTK_BOX (box), status_box, 
 			    FALSE, FALSE, GNOME_PAD);
 
-	label = gtk_label_new(_("Flags:"));
-	gtk_box_pack_start (GTK_BOX (status_box), label, 
-			    FALSE, FALSE, 0); 
-	
-	flabel = gtk_label_new ("0");
+	flabel = gtk_label_new ("");
 	gtk_box_pack_start (GTK_BOX (status_box), flabel, 
 			    FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Time:"));
-	gtk_box_pack_start (GTK_BOX (status_box), label, 
+	box = gtk_hbox_new (FALSE, 0);
+	label = gtk_label_new (_("Time: "));
+	gtk_box_pack_start (GTK_BOX (box), label, 
 	FALSE, FALSE, 0);
 
         clk = games_clock_new ();
-	gtk_box_pack_start (GTK_BOX (status_box), clk, 
+	gtk_box_pack_start (GTK_BOX (box), clk, 
 			    FALSE, FALSE, 0); 
 
-	update_score_state ();
+	gtk_box_pack_start (GTK_BOX (status_box), box,
+			    FALSE, FALSE, 0);
 
-	new_game (mfield, NULL);
+	new_game ();
 
         gtk_widget_show_all (window);
 
@@ -1105,7 +1077,7 @@ main (int argc, char *argv[])
 
 	/* All this hiding is a bit ugly, but it's better than a
 	 * ton of gtk_widget_show calls. */
-	gtk_widget_hide (ralign);
+	gtk_widget_hide (resume_container);
 	gtk_widget_hide (pm_win);
 	gtk_widget_hide (pm_sad);
 	gtk_widget_hide (pm_cool);
